import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:tengits_widgets/chart/chart.dart';

class TLChartBar extends StatefulWidget {
  const TLChartBar({
    Key? key,
    this.chartLabelMargin,
    this.chartLabelItemPadding,
    required this.groupDatas,
  }) : super(key: key);

  final EdgeInsetsGeometry? chartLabelMargin;

  final EdgeInsetsGeometry? chartLabelItemPadding;

  final List<TLBarChartLocalGroupData> groupDatas;

  @override
  State<TLChartBar> createState() => _TLChartBarState();
}

class _TLChartBarState extends State<TLChartBar> {
  ///
  late EdgeInsetsGeometry? _chartLabelMargin;

  ///
  late EdgeInsetsGeometry? _labelItemPadding;

  ///
  late List<TLBarChartLocalGroupData> _groupDatas;

  ///
  late Map tooltipIndexMap;

  ///
  Map get tooltipNormalInfoMap {
    int groupIndex = -1;
    int rodIndex = -1;
    double maxValue = 0;
    for (int i = 0; i < _groupDatas.length; i++) {
      TLBarChartLocalGroupData groupData = _groupDatas[i];
      List<TLBarChartLocalRodData> rodDatas = groupData.rodDatas;
      for (int j = 0; j < rodDatas.length; j++) {
        TLBarChartLocalRodData rodData = rodDatas[j];
        List<TLBarChartLocalRodStackItem> rodStackItems = rodData.rodStackItems ?? [];
        double sum = 0;
        for (int k = 0; k < rodStackItems.length; k++) {
          TLBarChartLocalRodStackItem rodStackItem = rodStackItems[k];
          sum += rodStackItem.value;
        }
        if (sum > maxValue) {
          maxValue = sum;
          groupIndex = i;
          rodIndex = j;
        }
      }
    }
    return {
      "groupIndex": groupIndex,
      "rodIndex": rodIndex,
    };
  }

  @override
  void initState() {
    ///
    _chartLabelMargin = widget.chartLabelMargin;

    ///
    _labelItemPadding = widget.chartLabelItemPadding;

    ///
    _groupDatas = widget.groupDatas;
    tooltipIndexMap = tooltipNormalInfoMap;

    super.initState();
  }

  @override
  void didUpdateWidget(TLChartBar oldWidget) {
    ///
    if (oldWidget.chartLabelMargin != widget.chartLabelMargin) {
      ///
      _chartLabelMargin = widget.chartLabelMargin;
    }

    ///
    if (oldWidget.chartLabelItemPadding != widget.chartLabelItemPadding) {
      ///
      _labelItemPadding = widget.chartLabelItemPadding;
    }

    ///
    if (oldWidget.groupDatas != widget.groupDatas) {
      ///
      _groupDatas = widget.groupDatas;
      tooltipIndexMap = tooltipNormalInfoMap;
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    TLChartMaxAndInterval mi = TLChartCalculateUtils.calculateChart(
      _groupDatas,
      (TLBarChartLocalGroupData groupData) {
        double maxValue = 0;
        List<TLBarChartLocalRodData> rodDatas = groupData.rodDatas;
        for (int j = 0; j < rodDatas.length; j++) {
          TLBarChartLocalRodData rodData = rodDatas[j];
          if (rodData.value != null) {
            return rodData.value!;
          }
          List<TLBarChartLocalRodStackItem> rodStackItems = rodData.rodStackItems ?? [];
          double sum = 0;
          for (int k = 0; k < rodStackItems.length; k++) {
            TLBarChartLocalRodStackItem rodStackItem = rodStackItems[k];
            sum += rodStackItem.value;
          }
          if (sum > maxValue) {
            maxValue = sum;
          }
        }
        return maxValue;
      },
      30,
      6,
    );
    return Container(
      alignment: Alignment.center,
      color: Colors.transparent,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Container(
            color: Colors.transparent,
            constraints: BoxConstraints(maxWidth: 800, maxHeight: 220.h),
            child: BarChart(
              BarChartData(
                maxY: mi.maxY.toDouble(),
                minY: 0,
                alignment: BarChartAlignment.spaceBetween,
                barTouchData: BarTouchData(
                  touchCallback: (touchEvent, response) {
                    if (response?.spot != null) {
                      if (mounted) {
                        setState(() {
                          tooltipIndexMap = {
                            "groupIndex": response?.spot?.touchedBarGroupIndex ?? -1,
                            "rodIndex": response?.spot?.touchedRodDataIndex ?? -1,
                          };
                        });
                      }
                    }
                  },
                  handleBuiltInTouches: true,
                  enabled: true,
                  touchTooltipData: BarTouchTooltipData(
                    tooltipBgColor: TLChartCommonStyle.toolTipBgColor,
                    tooltipMargin: 0,
                    fitInsideHorizontally: true,
                    fitInsideVertically: true,
                    tooltipPadding: const EdgeInsets.symmetric(horizontal: 8, vertical: 8),
                    getTooltipItem: (
                      BarChartGroupData group,
                      int groupIndex,
                      BarChartRodData rod,
                      int rodIndex,
                    ) {
                      String tip = "";
                      TLBarChartLocalRodData rodData = _groupDatas[groupIndex].rodDatas[rodIndex];
                      if (rodData.value != null) {
                        tip = rod.toY.toString().replaceAll(RegExp(r"([.]*0)(?!.*\d)"), "");
                      } else {
                        List<TLBarChartLocalRodStackItem> rodStackItems = rodData.rodStackItems ?? [];
                        if (rodStackItems.isNotEmpty) {
                          for (int i = 0; i < rodStackItems.length; i++) {
                            if (i > 0) {
                              tip += "\n";
                            }
                            tip += rodStackItems[i].rodStackItemTitle + "：" + (rodStackItems[i].value).toStringAsFixed(0);
                          }
                          tip += "\n总数：" + rod.toY.toString().replaceAll(RegExp(r"([.]*0)(?!.*\d)"), "");
                        }
                      }

                      return TLChartCommonStyle.getBarTooltipItem(tip, color: Colors.white);
                    },
                  ),
                ),
                borderData: TLChartCommonStyle.flBorderData,
                gridData: TLChartCommonStyle.getGridData(
                  mi.interval.toDouble(),
                ),
                barGroups: _barGroups(_groupDatas),
                titlesData: FlTitlesData(
                  show: true,
                  leftTitles: AxisTitles(
                    sideTitles: SideTitles(
                      showTitles: true,
                      interval: mi.interval.toDouble(),
                      reservedSize: 24.w,
                      getTitlesWidget: (double value, TitleMeta meta) {
                        return TLChartCommonStyle.getLeftTitleWidget("${value.toInt()}", meta);
                      },
                    ),
                  ),
                  topTitles: AxisTitles(sideTitles: SideTitles(showTitles: false)),
                  rightTitles: AxisTitles(sideTitles: SideTitles(showTitles: false)),
                  bottomTitles: AxisTitles(
                    sideTitles: SideTitles(
                      showTitles: true,
                      interval: 1,
                      getTitlesWidget: (value, titleMeta) {
                        List<TLBarChartLocalGroupData> datas = _groupDatas.where((e) => e.id == value.toInt()).toList();
                        String name = "";
                        if (datas.isNotEmpty) {
                          name = datas[0].title.replaceAll("公司", "");
                          int index = name.indexOf(RegExp(r'[Kk]'));
                          if (index <= 0) {
                            index = name.length;
                            name = name.substring(0, index);
                          }
                        }
                        return TLChartCommonStyle.getBottomTitleWidget(name, titleMeta);
                      },
                    ),
                  ),
                ),
              ),
            ),
          ),
          Visibility(
            visible: _groupDatas.isNotEmpty && _groupDatas[0].rodDatas.isNotEmpty,
            child: TLChartLabel(
              margin: _chartLabelMargin,
              labelItemPadding: _labelItemPadding,
              datas: (_groupDatas[0].rodDatas[0].rodStackItems ?? [])
                  .map((e) => TLChartLabelModel(
                        labelColor: e.color,
                        labelText: e.rodStackItemTitle,
                      ))
                  .toList(),
            ),
          ),
        ],
      ),
    );
  }

  List<BarChartGroupData> _barGroups(List<TLBarChartLocalGroupData> groupDatas) {
    List<BarChartGroupData> barChartGroupDatas = [];
    for (TLBarChartLocalGroupData groupData in groupDatas) {
      List<BarChartRodData> barChartRodData = [];
      for (TLBarChartLocalRodData rodData in groupData.rodDatas) {
        double sum = 0;
        List<BarChartRodStackItem> barChartRodStackItems = [];
        if (rodData.value == null) {
          for (TLBarChartLocalRodStackItem rodStackItem in rodData.rodStackItems ?? []) {
            if (rodStackItem.value > 0) {
              barChartRodStackItems.add(
                BarChartRodStackItem(
                  sum,
                  sum + rodStackItem.value,
                  rodStackItem.color,
                ),
              );
              sum += rodStackItem.value;
            }
          }
        }

        barChartRodData.add(
          BarChartRodData(
            toY: rodData.value?.toDouble() ?? sum,
            color: rodData.color,
            width: 16.w,
            borderRadius: BorderRadius.vertical(top: Radius.circular(2.r)),
            rodStackItems: barChartRodStackItems,
          ),
        );
      }
      barChartGroupDatas.add(
        BarChartGroupData(
          x: groupData.id,
          barRods: barChartRodData,
          barsSpace: 4,
          showingTooltipIndicators: tooltipIndexMap["groupIndex"] == groupDatas.indexOf(groupData) ? [tooltipIndexMap["rodIndex"]] : [],
        ),
      );
    }
    return barChartGroupDatas;
  }
}

class TLBarChartLocalGroupData {
  TLBarChartLocalGroupData({
    required this.id,
    required this.title,
    required this.rodDatas,
  });

  /// 唯一性id
  int id;

  ///
  String title;

  ///
  List<TLBarChartLocalRodData> rodDatas;
}

class TLBarChartLocalRodData {
  TLBarChartLocalRodData({
    this.value,
    this.color,
    this.rodStackItems,
  });

  num? value;
  Color? color;

  List<TLBarChartLocalRodStackItem>? rodStackItems;
}

class TLBarChartLocalRodStackItem {
  TLBarChartLocalRodStackItem({
    required this.rodStackItemTitle,
    required this.value,
    required this.color,
  });

  String rodStackItemTitle;
  num value;
  Color color;
}
