import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

class TLChartLabelModel {
  TLChartLabelModel({
    this.labelColor,
    this.labelText,
    this.labelTextStyle,
  });

  final Color? labelColor;
  final String? labelText;
  final TextStyle? labelTextStyle;
}

class TLChartLabel extends StatefulWidget {
  const TLChartLabel({
    Key? key,
    this.margin,
    this.labelItemPadding,
    required this.datas,
  }) : super(key: key);

  ///
  final EdgeInsetsGeometry? margin;

  ///
  final EdgeInsetsGeometry? labelItemPadding;

  ///
  final List<TLChartLabelModel> datas;

  @override
  State<TLChartLabel> createState() => _TLChartLabelState();
}

class _TLChartLabelState extends State<TLChartLabel> {
  ///
  late EdgeInsetsGeometry? _margin;

  ///
  late EdgeInsetsGeometry _labelItemPadding;

  ///
  late List<TLChartLabelModel> _datas;

  @override
  void initState() {
    ///
    _margin = widget.margin;

    ///
    _labelItemPadding = widget.labelItemPadding ?? _defaultItemPadding;

    ///
    _datas = widget.datas;

    super.initState();
  }

  @override
  void didUpdateWidget(TLChartLabel oldWidget) {
    ///
    if (oldWidget.margin != widget.margin) {
      ///
      _margin = widget.margin;
    }

    ///
    if (oldWidget.labelItemPadding != widget.labelItemPadding) {
      ///
      _labelItemPadding = widget.labelItemPadding ?? _defaultItemPadding;
    }

    ///
    if (oldWidget.datas != widget.datas) {
      ///
      _datas = widget.datas;
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (BuildContext context, BoxConstraints constraints) {
      List<Size> itemSizes = _datas.map(_obtainTextSize).toList();
      final maxSize = itemSizes.fold<Size>(Size.zero, (previousValue, element) {
        return previousValue.width.compareTo(element.width) >= 1 ? previousValue : element;
      });
      Size itemSize = Size(
        maxSize.width + _labelItemPadding.horizontal + 12.r, //12.r是颜色标签尺寸宽度
        maxSize.height + _labelItemPadding.vertical,
      );
      int rowNum = (constraints.maxWidth / itemSize.width).truncate(); //向下取整(返回int)
      int colNum = rowNum > 0 ? (_datas.length / rowNum).ceil() : 0; //向上取整（返回int）
      Size containerSize = Size(
        constraints.maxWidth,
        colNum > 0 ? itemSize.height * colNum + 8.h * (colNum - 1) : 0,
      );
      return Container(
        margin: colNum > 0 ? _margin ?? EdgeInsets.only(top: 12.h, left: 12.w) : EdgeInsets.zero,
        color: Colors.transparent,
        width: containerSize.width,
        height: containerSize.height,
        child: Wrap(
          direction: Axis.vertical,
          alignment: WrapAlignment.start,
          spacing: 8.h,
          runAlignment: WrapAlignment.spaceEvenly,
          children: _datas
              .map((e) => Container(
                    color: Colors.transparent,
                    height: itemSize.height,
                    width: itemSize.width,
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.start,
                      mainAxisSize: MainAxisSize.min,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        Container(
                          width: 12.r,
                          height: 12.r,
                          decoration: BoxDecoration(
                            shape: BoxShape.rectangle,
                            color: e.labelColor ?? Colors.grey,
                            borderRadius: BorderRadius.circular(2.r),
                          ),
                        ),
                        Container(
                          padding: _labelItemPadding,
                          child: Text(
                            e.labelText ?? "--",
                            style: _defaultTextStyle.merge(e.labelTextStyle),
                            maxLines: 1,
                            textDirection: TextDirection.ltr,
                          ),
                        ),
                      ],
                    ),
                  ))
              .toList(),
        ),
      );
    });
  }

  Size _obtainTextSize(TLChartLabelModel model) {
    final textPainter = TextPainter(
      text: TextSpan(
        text: model.labelText ?? "text",
        style: _defaultTextStyle.merge(model.labelTextStyle),
      ),
      maxLines: 1,
      textDirection: TextDirection.ltr,
    )..layout(
        minWidth: 0,
        maxWidth: double.infinity,
      );

    return textPainter.size;
  }

  ///
  final TextStyle _defaultTextStyle = TextStyle(color: const Color(0xFF323233), fontSize: 13.sp, fontWeight: FontWeight.w400);
  final EdgeInsetsGeometry _defaultItemPadding = EdgeInsets.symmetric(horizontal: 4.w);
}
