import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:tengits_widgets/chart/chart.dart';

class TLChartPie extends StatefulWidget {
  const TLChartPie({
    Key? key,
    this.centerTitle,
    this.tipUnit,
    this.chartLabelMargin,
    this.chartLabelItemPadding,
    required this.datas,
  }) : super(key: key);

  final String? centerTitle;
  final String? tipUnit;

  final EdgeInsetsGeometry? chartLabelMargin;

  final EdgeInsetsGeometry? chartLabelItemPadding;

  final List<PieChartDataModel> datas;

  @override
  State<TLChartPie> createState() => _TLChartPieState();
}

class _TLChartPieState extends State<TLChartPie> {
  ///
  late String _centerTitle;

  ///
  late String _tipUnit;

  ///
  late EdgeInsetsGeometry? _chartLabelMargin;

  ///
  late EdgeInsetsGeometry? _labelItemPadding;

  ///
  late List<PieChartDataModel> _datas;

  Offset? _localPosition;
  int _touchedIndex = -1;
  PieChartSectionData? _pieTouchedSectionData;

  @override
  void initState() {
    ///
    _centerTitle = widget.centerTitle ?? "";

    ///
    _tipUnit = widget.tipUnit ?? "";

    ///
    _chartLabelMargin = widget.chartLabelMargin;

    ///
    _labelItemPadding = widget.chartLabelItemPadding;

    ///
    _datas = widget.datas;

    super.initState();
  }

  @override
  void didUpdateWidget(TLChartPie oldWidget) {
    ///
    if (oldWidget.centerTitle != widget.centerTitle) {
      ///
      _centerTitle = widget.centerTitle ?? "";
    }

    ///
    if (oldWidget.tipUnit != widget.tipUnit) {
      ///
      _tipUnit = widget.tipUnit ?? "";
    }

    ///
    if (oldWidget.chartLabelMargin != widget.chartLabelMargin) {
      ///
      _chartLabelMargin = widget.chartLabelMargin;
    }

    ///
    if (oldWidget.chartLabelItemPadding != widget.chartLabelItemPadding) {
      ///
      _labelItemPadding = widget.chartLabelItemPadding;
    }

    ///
    if (oldWidget.datas != widget.datas) {
      ///
      _datas = widget.datas;
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      alignment: AlignmentDirectional.center,
      clipBehavior: Clip.none,
      children: [
        Container(
          alignment: Alignment.center,
          color: Colors.transparent,
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              Stack(
                alignment: AlignmentDirectional.center,
                children: [
                  Container(
                    color: Colors.transparent,
                    constraints: BoxConstraints(maxWidth: 160.r, maxHeight: 160.r),
                    child: PieChart(
                      PieChartData(
                        centerSpaceRadius: 60.r,
                        sectionsSpace: 0,
                        sections: _sections(_datas),
                        pieTouchData: PieTouchData(
                          enabled: true,
                          touchCallback: (FlTouchEvent event, pieTouchResponse) {
                            if (mounted) {
                              setState(() {
                                if (!event.isInterestedForInteractions ||
                                    pieTouchResponse == null ||
                                    pieTouchResponse.touchedSection == null ||
                                    (pieTouchResponse.touchedSection?.touchedSection?.title.isEmpty ?? true)) {
                                  _localPosition = null;
                                  _touchedIndex = -1;
                                  _pieTouchedSectionData = null;
                                  return;
                                }
                                _touchedIndex = pieTouchResponse.touchedSection!.touchedSectionIndex;
                                _pieTouchedSectionData = pieTouchResponse.touchedSection?.touchedSection;
                                _localPosition = event.localPosition;
                              });
                            }
                          },
                        ),
                      ),
                    ),
                  ),
                  SizedBox(
                    height: 120.h,
                    width: 120.w,
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.center,
                      mainAxisSize: MainAxisSize.min,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        Text(
                          _datas.map((e) => e.count).fold<num>(0, (previousValue, element) => previousValue + element).toStringAsFixed(2),
                          style: TextStyle(color: const Color(0xff323233), fontSize: 36.sp),
                          overflow: TextOverflow.ellipsis,
                          maxLines: 1,
                        ),
                        Visibility(
                          visible: _centerTitle.isNotEmpty,
                          child: Text(
                            _centerTitle,
                            style: TextStyle(color: const Color(0xff646566), fontSize: 13.sp),
                            overflow: TextOverflow.ellipsis,
                            maxLines: 2,
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
              TLChartLabel(
                margin: _chartLabelMargin,
                labelItemPadding: _labelItemPadding,
                datas: _datas
                    .map((e) => TLChartLabelModel(
                          labelColor: e.color,
                          labelText: e.text,
                          labelTextStyle: e.textStyle,
                        ))
                    .toList(),
              ),
            ],
          ),
        ),
        Visibility(
          visible: _datas.isNotEmpty && _localPosition != null && _touchedIndex >= 0 && _pieTouchedSectionData != null && (_pieTouchedSectionData?.title.isNotEmpty ?? false),
          child: Positioned(
            // left: _localPosition?.dx,
            // top: (_localPosition?.dy ?? 16.h) - 16.h,
            left: 0,
            top: 0,
            child: Container(
              padding: EdgeInsets.all(8.r),
              constraints: BoxConstraints(minWidth: 80.w, minHeight: 32.h, maxHeight: 32.h),
              decoration: BoxDecoration(color: Colors.black.withOpacity(0.8), borderRadius: BorderRadius.circular(3.r)),
              child: Text(
                "${_pieTouchedSectionData?.title}：${_pieTouchedSectionData?.value.toString().replaceAll(RegExp(r"([.]*0)(?!.*\d)"), "")}$_tipUnit",
                style: TextStyle(color: Colors.white, fontSize: 13.sp, fontWeight: FontWeight.w500),
                textAlign: TextAlign.center,
              ),
            ),
          ),
        ),
      ],
    );
  }

  List<PieChartSectionData> _sections(List<PieChartDataModel> datas) {
    List<PieChartSectionData> ret = [];
    for (int i = 0; i < datas.length; i++) {
      PieChartDataModel data = datas[i];
      if (data.count > 0) {
        ret.add(PieChartSectionData(
          value: data.count.toDouble(),
          color: data.color,
          title: data.text,
          titleStyle: data.textStyle ?? TextStyle(color: const Color(0xFF323233), fontSize: 14.sp, fontWeight: FontWeight.bold),
          showTitle: data.showTextTitle,
          radius: 20.r,
        ));
      }
    }
    if (ret.isEmpty) {
      ret.add(PieChartSectionData(
        value: 1,
        color: const Color(0xFFDCDEE0),
        showTitle: false,
        radius: 20.r,
      ));
    }
    return ret;
  }
}

class PieChartDataModel {
  PieChartDataModel({
    this.count = 0,
    this.color,
    this.text,
    this.textStyle,
    this.showTextTitle = false,
  });

  final num count;
  final Color? color;
  final String? text;
  final TextStyle? textStyle;
  final bool showTextTitle;
}
