import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:tengits_widgets/tengits_widgets.dart';

class TLListItem extends StatefulWidget {
  const TLListItem({
    Key? key,
    this.leadingTitle,
    this.contentValue,
    this.listItemStyle = const TLListItemStyle(),
    this.listItemBuilders,
    this.listItemEvent,
  }) : super(key: key);

  /// 左标题
  final String? leadingTitle;

  /// 文本默认值
  final String? contentValue;

  /// 风格配置
  final TLListItemStyle listItemStyle;

  /// 自定义Builders集
  final TLListItemBuilders? listItemBuilders;

  /// Builders集 手势事件
  final TLListItemEvent? listItemEvent;

  @override
  State<TLListItem> createState() => _TLListItemState();
}

class _TLListItemState extends State<TLListItem> {
  ///
  late String? _leadingTitle;
  late String? _contentValue;
  late TLListItemStyle _listItemStyle;
  late TLListItemBuilders? _listItemBuilders;
  late TLListItemEvent? _listItemEvent;

  @override
  void initState() {
    ///
    _leadingTitle = widget.leadingTitle;

    ///
    _contentValue = widget.contentValue;

    ///
    _listItemStyle = widget.listItemStyle;

    ///
    _listItemBuilders = widget.listItemBuilders;

    ///
    _listItemEvent = widget.listItemEvent;

    super.initState();
  }

  @override
  void didUpdateWidget(TLListItem oldWidget) {
    ///
    if (oldWidget.leadingTitle != widget.leadingTitle) {
      ///
      _leadingTitle = widget.leadingTitle;
    }

    ///
    if (oldWidget.contentValue != widget.contentValue) {
      ///
      _contentValue = widget.contentValue;
    }

    ///
    if (oldWidget.listItemStyle != widget.listItemStyle) {
      ///
      _listItemStyle = widget.listItemStyle;
    }

    ///
    if (oldWidget.listItemBuilders != widget.listItemBuilders) {
      ///
      _listItemBuilders = widget.listItemBuilders;
    }

    ///
    if (oldWidget.listItemEvent != widget.listItemEvent) {
      ///
      _listItemEvent = widget.listItemEvent;
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: _listItemEvent?.onTap,
      child: Container(
        color: Colors.transparent,
        child: Column(
          children: [
            Row(
              mainAxisAlignment: MainAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Padding(
                  padding: EdgeInsets.only(top: 3.h),
                  child: Text(
                    '*',
                    style: _listItemStyle.isRequire
                        ? TextStyle(color: Colors.red, fontSize: 16.sp)
                        : const TextStyle(
                            color: Colors.transparent,
                          ),
                  ),
                ),
                Visibility(
                  visible: _listItemStyle.showLeading,
                  child: Expanded(
                    child: GestureDetector(
                      onTap: _listItemEvent?.onLeadingTap,
                      child: _listItemBuilders?.leading?.call(context) ??
                          Padding(
                            padding: EdgeInsets.only(top: 3.h),
                            child: Text(
                              _leadingTitle ?? "",
                              style: TextStyle(color: const Color(0xFF323233), fontSize: 14.sp, height: 1.5),
                              textAlign: TextAlign.left,
                              softWrap: true,
                              overflow: TextOverflow.ellipsis,
                              maxLines: _listItemStyle.leadingTextMaxLines,
                            ),
                          ),
                    ),
                    flex: 3,
                  ),
                ),
                Expanded(
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.start,
                    mainAxisSize: MainAxisSize.min,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Expanded(
                        child: _listItemBuilders?.content?.call(context) ??
                            TLTextField(
                              defaultValue: _contentValue,
                              textFieldStyle: TLTextFieldStyle(
                                style: TextStyle(color: const Color(0xFF323233), fontSize: 15.sp, height: 1.5),
                                decoration: TLInputDecoration.spaceInputBorder(
                                  hintText: _listItemStyle.contentHintText,
                                  hintStyle: TextStyle(color: const Color(0xFF969799), fontSize: 16.sp, height: 1.5),
                                  contentPadding: EdgeInsets.only(left: 4.h),
                                ).decoration,
                                inputFormatters: _listItemStyle.inputFormatters ?? TLTextInputFormatter.maxLengthInputFormatters(maxLength: _listItemStyle.maxLength),
                                enabled: _listItemStyle.enabled,
                                keyboardType: _listItemStyle.keyboardType,
                              ),
                              textFieldEvent: _listItemEvent?.textFieldEvent,
                            ),
                      ),
                      Visibility(
                        visible: _listItemStyle.showTrailing,
                        child: GestureDetector(
                          onTap: _listItemEvent?.onTrailingTap,
                          child: _listItemBuilders?.trailing?.call(context) ??
                              Padding(
                                padding: EdgeInsets.only(left: 4.w, top: 6.h, right: 4.w, bottom: 4.h),
                                child: _listItemStyle.trailingIcon ??
                                    Icon(
                                      _listItemStyle.trailingIconData ?? Icons.keyboard_arrow_down,
                                      color: _listItemStyle.trailingIconColor,
                                      size: 16.r,
                                    ),
                              ),
                        ),
                      ),
                    ],
                  ),
                  flex: 7,
                ),
              ],
            ),
            Visibility(
              visible: _listItemStyle.showDivider,
              child: Divider(
                height: 30.h,
                thickness: 1.h,
                color: _listItemStyle.dividerColor,
              ),
            ),
          ],
        ),
      ),
    );
  }
}
