import 'package:flutter/material.dart';

import 'selection_common_entity.dart';

class SelectionUtil{
  static bool isEmpty(Object? obj) {
    if (obj is String) {
      return obj.isEmpty;
    }
    if (obj is Iterable) {
      return obj.isEmpty;
    }
    if (obj is Map) {
      return obj.isEmpty;
    }
    return obj == null;
  }

  static Size textSize(String text, TextStyle style) {
    if (isEmpty(text)) return const Size(0, 0);
    final TextPainter textPainter = TextPainter(
      text: TextSpan(text: text, style: style),
      maxLines: 1,
      textDirection: TextDirection.ltr,
    )..layout(minWidth: 0, maxWidth: double.infinity);
    return textPainter.size;
  }

  static void processBrotherItemSelectStatus(
      TSelectionEntity selectionEntity) {
    if (TSelectionFilterType.checkbox == selectionEntity.filterType) {
      selectionEntity.isSelected = !selectionEntity.isSelected;
      List<TSelectionEntity>? allBrothers = selectionEntity.parent?.children;
      if (!isEmpty(allBrothers)) {
        for (TSelectionEntity entity in allBrothers!) {
          if (entity != selectionEntity) {
            if (entity.filterType == TSelectionFilterType.radio) {
              entity.isSelected = false;
            }

            if (entity.filterType == TSelectionFilterType.date) {
              entity.isSelected = false;
              entity.value = null;
            }
          }
        }
      }
    }
    if (TSelectionFilterType.radio == selectionEntity.filterType) {
      selectionEntity.parent?.clearChildSelection();
      selectionEntity.isSelected = true;
    }

    if (TSelectionFilterType.date == selectionEntity.filterType) {
      selectionEntity.parent?.clearChildSelection();

      /// 日期类型时在外部 Picker 点击确定时设置 选中状态
      selectionEntity.isSelected = true;
    }
  }

  static List<TSelectionEntity> currentSelectListForEntity(
      TSelectionEntity entity) {
    List<TSelectionEntity> list = [];
    for (TSelectionEntity entity in entity.children) {
      if (entity.isSelected) {
        list.add(entity);
      }
    }
    return list;
  }

    /// 确定当前 Item 在第几层级
  static int getCurrentListIndex(TSelectionEntity? currentItem) {
    int listIndex = -1;
    if (currentItem != null) {
      listIndex = 0;
      var parent = currentItem.parent;
      while (parent != null) {
        listIndex++;
        parent = parent.parent;
      }
    }
    return listIndex;
  }

    /// 判断列表中是否有range类型
  static bool hasRangeItem(List<TSelectionEntity> list) {
    for (TSelectionEntity entity in list) {
      if (TSelectionFilterType.range == entity.filterType ||
          TSelectionFilterType.dateRange == entity.filterType ||
          TSelectionFilterType.dateRangeCalendar == entity.filterType ||
          TSelectionFilterType.range == entity.filterShowType) {
        return true;
      }
    }
    return false;
  }

  /// 判断列表中是否有range类型
  static TSelectionEntity? getFilledCustomInputItem(
      List<TSelectionEntity> list) {
    TSelectionEntity? filledCustomInputItem;
    for (TSelectionEntity entity in list) {
      if (entity.isSelected &&
          (TSelectionFilterType.range == entity.filterType ||
              TSelectionFilterType.dateRange == entity.filterType ||
              TSelectionFilterType.dateRangeCalendar == entity.filterType) &&
          entity.customMap != null) {
        filledCustomInputItem = entity;
        break;
      }
      if (entity.children.isNotEmpty) {
        filledCustomInputItem = getFilledCustomInputItem(entity.children);
      }
      if (filledCustomInputItem != null) {
        break;
      }
    }
    return filledCustomInputItem;
  }

    /// 筛选项最多不超过三层,故直接写代码判断,本质为深度优先搜索。
  static int getTotalLevel(TSelectionEntity entity) {
    int level = 0;
    TSelectionEntity rootEntity = entity;
    while (rootEntity.parent != null) {
      rootEntity = rootEntity.parent!;
    }

    if (rootEntity.children.isNotEmpty) {
      level = level > 1 ? level : 1;
      for (TSelectionEntity firstLevelEntity in rootEntity.children) {
        if (firstLevelEntity.children.isNotEmpty) {
          level = level > 2 ? level : 2;
          for (TSelectionEntity secondLevelEntity
              in firstLevelEntity.children) {
            if (secondLevelEntity.children.isNotEmpty) {
              level = 3;
              break;
            }
          }
        }
      }
    }
    return level;
  }

  static bool checkMaxSelectionCount(TSelectionEntity entity) {
    return entity.getLimitedRootSelectedChildCount() <
        entity.getLimitedRootMaxSelectedCount();
  }

  //设置数据为未选中状态
  static void resetSelectionDatas(TSelectionEntity entity) {
    entity.isSelected = false;
    entity.customMap = {};
    for (TSelectionEntity subEntity in entity.children) {
      resetSelectionDatas(subEntity);
    }
  }

}