import 'dart:async';

class TEventBus {
  TEventBus({
    bool sync = false,
  }) : _streamController = StreamController.broadcast(sync: sync);

  TEventBus.customController(
    StreamController controller,
  ) : _streamController = controller;

  static TEventBus? _instance;

  factory TEventBus._() {
    _instance ??= TEventBus();
    return _instance!;
  }

  static TEventBus get instance {
    return TEventBus._();
  }

  StreamController _streamController;

  /// Controller for the event bus stream.
  StreamController get streamController => _streamController;

  ///
  Stream<T> on<T>() {
    if (T == dynamic) {
      return streamController.stream as Stream<T>;
    }
    return streamController.stream.where((dynamic e) => e is T).cast<T>();
  }

  /// Fires a new event on the event bus with the specified [event].
  void fire(event) {
    streamController.add(event);
  }

  /// Destroy this [TEventBus]. This is generally only in a testing context.
  void destroy() {
    _streamController.close();
  }
}
