import 'package:flutter/material.dart';
import 'package:tengits_widgets/selection/selection_common_entity.dart';
import 'package:tengits_widgets/selection/selection_utils.dart';
import 'package:tengits_widgets/selection/t_selection_config.dart';
import 'package:tengits_widgets/utils/image_utils.dart';

class TSelectionCommonItemWidget extends StatelessWidget {
  final TSelectionEntity item;
  final Color? backgroundColor;
  final Color? selectedBackgroundColor;
  final bool isCurrentFocused;
  final bool isFirstLevel;
  final bool isMoreSelectionListType;

  final ValueChanged<TSelectionEntity>? itemSelectFunction;

  final TSelectionConfig? themeData;

  const TSelectionCommonItemWidget({
    Key? key,
    required this.item,
    this.backgroundColor,
    this.isFirstLevel = false,
    this.isMoreSelectionListType = false,
    this.itemSelectFunction,
    this.selectedBackgroundColor,
    this.isCurrentFocused = false,
    this.themeData,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    Container checkbox;
    if (!item.isUnLimit() && (item.children.isEmpty)) {
      if (item.isInLastLevel() && item.hasCheckBoxBrother()) {
        checkbox = Container(
          padding: const EdgeInsets.only(left: 6),
          width: 21,
          child: (item.isSelected) ? ImageUtils.getAssetImageWithColor("images/icon_multi_selected.png", const Color(0xFF0984F9)) : ImageUtils.getAssetImage("images/icon_unselect.png"),
        );
      } else {
        checkbox = Container();
      }
    } else {
      checkbox = Container();
    }

    return GestureDetector(
      onTap: () {
        if (itemSelectFunction != null) {
          itemSelectFunction!(item);
        }
      },
      child: Container(
        padding: const EdgeInsets.only(left: 20, right: 20, top: 10, bottom: 10),
        color: getItemBGColor(),
        child: Align(
          alignment: Alignment.centerLeft,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              Row(
                children: <Widget>[
                  Expanded(
                    child: Text(
                      item.title + getSelectedItemCount(item),
                      softWrap: true,
                      overflow: TextOverflow.ellipsis,
                      textAlign: TextAlign.left,
                      style: getItemTextStyle(),
                    ),
                  ),
                  checkbox
                ],
              ),
              Visibility(
                visible: !SelectionUtil.isEmpty(item.subTitle),
                child: Padding(
                  padding: EdgeInsets.only(right: item.isInLastLevel() ? 21 : 0),
                  child: Text(item.subTitle ?? '', style: const TextStyle(fontSize: 12, fontWeight: FontWeight.normal, decoration: TextDecoration.none, color: Color(0xFF999999)), maxLines: 1, overflow: TextOverflow.ellipsis),
                ),
              )
            ],
          ),
        ),
      ),
    );
  }

  Color? getItemBGColor() {
    if (isCurrentFocused) {
      return selectedBackgroundColor;
    } else {
      return backgroundColor;
    }
  }

  bool isHighLight(TSelectionEntity item) {
    if (item.isInLastLevel()) {
      if (item.isUnLimit()) {
        return isCurrentFocused;
      } else {
        return item.isSelected;
      }
    } else {
      return isCurrentFocused;
    }
  }

  bool isBold(TSelectionEntity item) {
    if (isHighLight(item)) {
      return true;
    } else {
      return item.hasCheckBoxBrother() && item.selectedList().isNotEmpty;
    }
  }

  TextStyle? getItemTextStyle() {
    if (isHighLight(item)) {
      return const TextStyle(color: Color(0xFF0984F9), fontSize: 14.0, fontWeight: FontWeight.w600);
    } else if (isBold(item)) {
      return const TextStyle(color: Color(0xFF222222), fontSize: 14.0, fontWeight: FontWeight.w600);
    }
    return const TextStyle(color: Color(0xFF222222), fontSize: 14.0);
  }

  String getSelectedItemCount(TSelectionEntity item) {
    String itemCount = "";
    if ((SelectionUtil.getTotalLevel(item) < 3 || !isFirstLevel) && item.children.isNotEmpty) {
      int count = item.children.where((f) => f.isSelected && !f.isUnLimit()).length;
      if (count > 1) {
        return '($count)';
      } else if (count == 1 && item.hasCheckBoxBrother()) {
        return '($count)';
      } else {
        var unLimited = item.children.where((f) => f.isSelected && f.isUnLimit()).toList();
        if (unLimited.isNotEmpty) {
          return '(全部)';
        }
      }
    }
    return itemCount;
  }
}
