import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:tengits_widgets/calendar/table_calendar.dart';
import 'package:tengits_widgets/selection/selection_common_entity.dart';
import 'package:tengits_widgets/selection/selection_utils.dart';
import 'package:tengits_widgets/date_picker/t_date_time_formatter.dart';
import 'package:tengits_widgets/selection/t_selection_config.dart';
import 'package:tengits_widgets/selection/t_selection_view_date_picker_controller.dart';

import 't_selection_datepicker_animate_widget.dart';

const String _defaultDateFormat = 'yyyy年MM月dd日';

// ignore: must_be_immutable
class TSelectionDateRangeItemWidget extends StatefulWidget {
  final TSelectionEntity item;

  /// 输入框显示文字大小
  final double showTextSize;

  /// 是否需要标题
  final bool isNeedTitle;

  /// 日期格式
  final String dateFormat;

  final TextEditingController minTextEditingController;
  final TextEditingController maxTextEditingController;

  final VoidCallback? onTapped;

  TSelectionConfig themeData;

  TSelectionDateRangeItemWidget(
      {Key? key,
      required this.item,
      required this.minTextEditingController,
      required this.maxTextEditingController,
      this.isNeedTitle = true,
      this.showTextSize = 16,
      this.dateFormat = _defaultDateFormat,
      this.onTapped,
      required this.themeData})
      : super(key: key);

  @override
  _TSelectionDateRangeItemWidgetState createState() =>
      _TSelectionDateRangeItemWidgetState();
}

class _TSelectionDateRangeItemWidgetState
    extends State<TSelectionDateRangeItemWidget> {
  final TSelectionDatePickerController _datePickerController =
      TSelectionDatePickerController();

  @override
  void initState() {
    DateTime? minDateTime;
    if (widget.item.customMap != null &&
        widget.item.customMap!['min'] != null) {
      minDateTime = DateTimeFormatter.convertIntValueToDateTime(
          widget.item.customMap!['min']);
    }
    DateTime? maxDateTime;
    if (widget.item.customMap != null &&
        widget.item.customMap!['max'] != null) {
      maxDateTime = DateTimeFormatter.convertIntValueToDateTime(
          widget.item.customMap!['max']);
    }
    widget.minTextEditingController.text = minDateTime != null
        ? DateTimeFormatter.formatDate(
            minDateTime, widget.dateFormat)
        : '';
    widget.maxTextEditingController.text = maxDateTime != null
        ? DateTimeFormatter.formatDate(
            maxDateTime, widget.dateFormat)
        : '';
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.only(left: 20, right: 20, bottom: 20),
      child: Column(
        children: <Widget>[
          widget.isNeedTitle
              ? Container(
                  margin: const EdgeInsets.only(bottom: 5),
                  alignment: Alignment.centerLeft,
                  child: Text(
                    widget.item.title.isEmpty ? '自定义区间' : widget.item.title,
                    textAlign: TextAlign.left,
                    style: const TextStyle(color:  Color(0xFF222222), fontSize: 16.0, fontWeight: FontWeight.w600),
                  ),
                )
              : Container(),
          Row(
            children: <Widget>[
              getDateRangeTextField(false),
              const Text(
                '至',
                style: TextStyle(color:  Color(0xFF222222), fontSize: 14.0),
              ),
              getDateRangeTextField(true),
            ],
          )
        ],
      ),
    );
  }

  Widget getDateRangeTextField(bool isMax) {
    return Expanded(
      child: TextField(
        enableInteractiveSelection: false,
        readOnly: true,
        onTap: () {
          if (widget.onTapped != null) {
            widget.onTapped!();
          }
          onTextTapped(isMax);
        },
        style: const TextStyle(color:  Color(0xFF222222), fontSize: 14.0),
        inputFormatters: [FilteringTextInputFormatter.digitsOnly],
        keyboardType: const TextInputType.numberWithOptions(),
        onChanged: (input) {
          widget.item.isSelected = true;
        },
        controller: isMax
            ? widget.maxTextEditingController
            : widget.minTextEditingController,
        cursorColor: const Color(0xFF3072F6),
        textAlign: TextAlign.center,
        decoration: InputDecoration(
          hintStyle: const TextStyle(color:  Color(0xFFCCCCCC), fontSize: 14.0),
          hintText: (!isMax ? '开始日期' : '结束日期'),
          enabledBorder: const UnderlineInputBorder(
              borderSide: BorderSide(
            width: 1,
            color: Color(0xFFF0F0F0),
          )),
          focusedBorder: const UnderlineInputBorder(
              borderSide: BorderSide(
            width: 1,
            color: Color(0xFFF0F0F0),
          )),
          contentPadding: const EdgeInsets.all(0),
        ),
      ),
    );
  }

  void onTextTapped(bool isMax) {
    if (_datePickerController.isShow) return;
    // String format = 'yyyy年,MM月,dd日';
    // DateTime? minDate = DateTimeFormatter.convertIntValueToDateTime(
    //     (widget.item.extMap)['min']);
    // DateTime? maxDate = DateTimeFormatter.convertIntValueToDateTime(
    //     (widget.item.extMap)['max']);

    DateTime? minSelectedDateTime = SelectionUtil.isEmpty(widget.item.customMap)
        ? null
        : DateTimeFormatter.convertIntValueToDateTime(
            widget.item.customMap!['min']);
    DateTime? maxSelectedDateTime = SelectionUtil.isEmpty(widget.item.customMap)
        ? null
        : DateTimeFormatter.convertIntValueToDateTime(
            widget.item.customMap!['max']);

    //DateTime? _minDateTime;
    //DateTime? _maxDateTime;
    // if (widget.item.customMap == null ||
    //     (widget.item.customMap!['min'] == null &&
    //         widget.item.customMap!['max'] == null)) {
    //   // 如果开始时间和结束时间均未选择
    //   _minDateTime = minDate;
    //   _maxDateTime = maxDate;
    // } else {
    //   _minDateTime = !isMax ? minDate : (minSelectedDateTime ?? minDate);
    //   _maxDateTime = isMax ? maxDate : (maxSelectedDateTime ?? maxDate);
    // }

    Widget content = TableCalendar(
      locale: "zh",
      firstDay: DateTime(2000, 1, 1),
      lastDay: DateTime(2050, 12, 31),
      focusedDay: DateTime.now(),
      calendarFormat: CalendarFormat.month,
      rangeSelectionMode: RangeSelectionMode.toggledOn,
      rangeStartDay: minSelectedDateTime, 
      rangeEndDay: maxSelectedDateTime,
      onRangeSelected: (start, end, focusedDay) {
        widget.item.isSelected = true;
        widget.item.customMap ??= {};
        widget.item.customMap!['min'] =
            start?.millisecondsSinceEpoch.toString() ?? "";
        widget.item.customMap!['max'] =
            end?.millisecondsSinceEpoch.toString() ?? "";
        closeSelectionPopupWindow();

        if (!isMax &&
            SelectionUtil.isEmpty(widget.maxTextEditingController.text)) {
          onTextTapped(true);
        }
        setState(() {});
      },
    );

    OverlayEntry entry = _createDatePickerEntry(context, content);
    Overlay.of(context)?.insert(entry);
    _datePickerController.entry = entry;
    _datePickerController.show();
  }

  OverlayEntry _createDatePickerEntry(BuildContext context, Widget content) {
    return OverlayEntry(builder: (context) {
      return GestureDetector(
        onTap: () {
          closeSelectionPopupWindow();
        },
        child: Container(
          color: const Color(0xB3000000),
          height: MediaQuery.of(context).size.height,
          alignment: Alignment.bottomCenter,
          child: TSelectionDatePickerAnimationWidget(
              controller: _datePickerController, view: content),
        ),
      );
    });
  }

  void closeSelectionPopupWindow() {
    if (_datePickerController.isShow) {
      _datePickerController.hide();
    }
  }

  @override
  void dispose() {
    _datePickerController.hide();
    super.dispose();
  }
}
