import 'package:flutter/material.dart';
import 'package:tengits_widgets/selection/t_selection_config.dart';

// ignore: must_be_immutable
class TSelectionMenuItemWidget extends StatelessWidget {
  final String title;
  final bool isHighLight;
  final bool active;
  final VoidCallback? itemClickFunction;

  TSelectionConfig themeData;

  TSelectionMenuItemWidget(
      {Key? key,
      required this.title,
      this.isHighLight = false,
      this.active = false,
      this.itemClickFunction,
      required this.themeData})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Expanded(
      child: GestureDetector(
        onTap: () {
          _menuItemTapped();
        },
        child: Container(
          color: Colors.transparent,
          constraints: const BoxConstraints.expand(),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              Flexible(
                child: Text(
                title,
                textAlign: TextAlign.center,
                overflow: TextOverflow.ellipsis,
                maxLines: 1,
                softWrap: true,
                style: isHighLight
                    ? const TextStyle(color:  Color(0xFF0984F9), fontSize: 14.0, fontWeight: FontWeight.w600)
                    : const TextStyle(color:  Color(0xFF222222), fontSize: 14.0),
                  ),
              ),
              Padding(
                  padding: const EdgeInsets.only(left: 4),
                  child: isHighLight
                      ? (active
                          ? const Icon(Icons.expand_less, color: Color(0xFF0984F9), size: 12,)
                          : const Icon(Icons.expand_more, color: Color(0xFF0984F9), size: 12,))
                      : (active
                          ? const Icon(Icons.expand_less, color: Color(0xFF0984F9), size:12)
                          : const Icon(Icons.expand_more, size: 12,)))
            ],
          ),
        ),
      ),
      flex: 1,
    );
  }

  void _menuItemTapped() {
    if (itemClickFunction != null) {
      itemClickFunction!();
    }
  }
}
