import 'package:flutter/material.dart';
import 'package:tengits_widgets/selection/selection_common_entity.dart';
import 'package:tengits_widgets/selection/selection_utils.dart';
import 'package:tengits_widgets/date_picker/t_date_time_formatter.dart';
import 'package:tengits_widgets/selection/t_selection_config.dart';
import 'package:tengits_widgets/toast/t_toast.dart';

/// /// /// /// /// /// /// /// /// /
/// 描述: 多选 tag 组件
/// /// /// /// /// /// /// /// /// /
class TSelectionRangeTagWidget extends StatefulWidget {
  //tag 显示的文本
  @required
  final List<TSelectionEntity> tagFilterList;

  //初始选中的 Index 列表
  final List<bool>? initSelectStatus;

  //选择tag的回调
  final void Function(int, bool)? onSelect;
  final double spacing;
  final double verticalSpacing;
  final int tagWidth;
  final double tagHeight;
  final int initFocusedIndex;

  final TSelectionConfig themeData;

  const TSelectionRangeTagWidget(
      {Key? key,
      required this.tagFilterList,
      this.initSelectStatus,
      this.onSelect,
      this.spacing = 12,
      this.verticalSpacing = 10,
      this.tagWidth = 75,
      this.tagHeight = 34,
      required this.themeData,
      this.initFocusedIndex = -1})
      : super(key: key);

  @override
  _TSelectionRangeTagWidgetState createState() =>
      _TSelectionRangeTagWidgetState();
}

class _TSelectionRangeTagWidgetState
    extends State<TSelectionRangeTagWidget> {
  @override
  Widget build(BuildContext context) {
    return Wrap(
      runSpacing: widget.verticalSpacing,
      spacing: widget.spacing,
      children: _tagWidgetList(context),
    );
  }

  List<Widget> _tagWidgetList(context) {
    List<Widget> list = [];
    for (int nameIndex = 0;
        nameIndex < widget.tagFilterList.length;
        nameIndex++) {
      Widget tagWidget = _tagWidgetAtIndex(nameIndex);
      GestureDetector gdt = GestureDetector(
          child: tagWidget,
          onTap: () {
            var selectedEntity = widget.tagFilterList[nameIndex];
            if (TSelectionFilterType.checkbox == selectedEntity.filterType &&
                !selectedEntity.isSelected) {
              if (!SelectionUtil.checkMaxSelectionCount(selectedEntity)) {
                TToast.show('您选择的筛选条件数量已达上限', context);
                return;
              }
            }
            SelectionUtil.processBrotherItemSelectStatus(selectedEntity);
            if (null != widget.onSelect) {
              widget.onSelect!(nameIndex, selectedEntity.isSelected);
            }
            setState(() {});
          });
      list.add(gdt);
    }
    return list;
  }

  Widget _tagWidgetAtIndex(int nameIndex) {
    bool selected = widget.tagFilterList[nameIndex].isSelected ||
        nameIndex == widget.initFocusedIndex;
    String text = widget.tagFilterList[nameIndex].title;
    if (widget.tagFilterList[nameIndex].filterType ==
            TSelectionFilterType.date &&
        !SelectionUtil.isEmpty(widget.tagFilterList[nameIndex].value)) {
      if (int.tryParse(widget.tagFilterList[nameIndex].value ?? '') != null) {
        DateTime? dateTime = DateTimeFormatter.convertIntValueToDateTime(
            widget.tagFilterList[nameIndex].value);
        if (dateTime != null) {
          text = DateTimeFormatter.formatDate(
              dateTime, 'yyyy年MM月dd日');
        }
      } else {
        text = widget.tagFilterList[nameIndex].value ?? '';
      }
    }

    Text tx =
        Text(text, style: selected ? _selectedTextStyle() : _tagTextStyle());
    Container tagItem = Container(
      alignment: Alignment.center,
      decoration: BoxDecoration(
          color: selected
              ? widget.themeData.tagSelectedBackgroundColor
              : widget.themeData.tagNormalBackgroundColor,
          borderRadius: BorderRadius.circular(widget.themeData.tagRadius)),
      width: widget.tagWidth.toDouble(),
      height: widget.tagHeight,
      child: tx,
    );
    return tagItem;
  }

  TextStyle _tagTextStyle() {
    return const TextStyle(color:  Color(0xFF222222), fontSize: 12.0);
  }

  TextStyle _selectedTextStyle() {
    return const TextStyle(color:  Color(0xFF0984F9), fontSize: 12.0, fontWeight: FontWeight.w600);
  }
}
