import 'package:flutter/material.dart';

/// TabBar配置类
class TTabBarConfig {
  /// 遵循外部主题配置
  /// 默认为 [BrnDefaultConfigUtils.tabBarConfig]
  TTabBarConfig({
    double? tabHeight,
    double? indicatorHeight,
    double? indicatorWidth,
    TextStyle? labelStyle,
    TextStyle? unselectedLabelStyle,
    Color? backgroundColor,
    TextStyle? tagNormalTextStyle,
    Color? tagNormalBgColor,
    TextStyle? tagSelectedTextStyle,
    Color? tagSelectedBgColor,
    double? tagRadius,
    double? tagSpacing,
    int? preLineTagCount,
    double? tagHeight,
  })  : _tabHeight = tabHeight,
        _indicatorHeight = indicatorHeight,
        _indicatorWidth = indicatorWidth,
        _labelStyle = labelStyle,
        _unselectedLabelStyle = unselectedLabelStyle,
        _backgroundColor = backgroundColor,
        _tagNormalTextStyle = tagNormalTextStyle,
        _tagNormalBgColor = tagNormalBgColor,
        _tagSelectedTextStyle = tagSelectedTextStyle,
        _tagSelectedBgColor = tagSelectedBgColor,
        _tagRadius = tagRadius,
        _tagSpacing = tagSpacing,
        _preLineTagCount = preLineTagCount,
        _tagHeight = tagHeight;

  /// TabBar 的整体高度
  /// 默认为 50
  final double? _tabHeight;

  /// 指示器的高度
  /// 默认为 2
  final double? _indicatorHeight;

  /// 指示器的宽度
  /// 默认为 24
  final double? _indicatorWidth;

  /// 选中 Tab 文本的样式
  ///
  /// TextStyle(
  ///   color: [BrnCommonConfig.brandPrimary],
  ///   fontSize: [BrnCommonConfig.fontSizeSubHead],
  /// )
  final TextStyle? _labelStyle;

  /// 未选中 Tab 文本的样式
  ///
  /// TextStyle(
  ///   color: [BrnCommonConfig.colorTextBase],
  ///   fontSize: [BrnCommonConfig.fontSizeSubHead],
  /// )
  final TextStyle? _unselectedLabelStyle;

  /// 背景色
  /// 默认为 [BrnCommonConfig.fillBase]
  final Color? _backgroundColor;

  /// 标签字体样式
  ///
  /// TextStyle(
  ///   color: [BrnCommonConfig.colorTextBase],
  ///   fontSize: [BrnCommonConfig.fontSizeCaption],
  /// )
  final TextStyle? _tagNormalTextStyle;

  /// 标签背景色
  /// 默认为 [BrnCommonConfig.brandPrimary].withAlpha(0x14),
  final Color? _tagNormalBgColor;

  /// 标签字体样式
  ///
  /// TextStyle(
  ///   color:[BrnCommonConfig.brandPrimary],
  ///   fontSize: [BrnCommonConfig.fontSizeCaption],
  /// )
  final TextStyle? _tagSelectedTextStyle;

  /// 标签选中背景色
  /// 默认为 [BrnCommonConfig.fillBody]
  final Color? _tagSelectedBgColor;

  /// tag圆角
  /// 默认为 [BrnCommonConfig.radiusSm]
  final double? _tagRadius;

  /// tag间距
  /// 默认为 12
  final double? _tagSpacing;

  /// 每行的tag数
  /// 默认为 4
  final int? _preLineTagCount;

  /// tag高度
  /// 默认为 32
  final double? _tagHeight;

  double get tabHeight =>
      _tabHeight ?? 50.0;

  double get indicatorHeight =>
      _indicatorHeight ??
      2.0;

  double get indicatorWidth =>
      _indicatorWidth ??
      24.0;

  TextStyle get labelStyle =>
      _labelStyle ?? const TextStyle(
      color: Color(0xFF0984F9),
      fontSize: 16.0,
      fontWeight: FontWeight.w600,
    );

  TextStyle get unselectedLabelStyle =>
      _unselectedLabelStyle ??
      const TextStyle(
      color: Color(0xFF222222),
      fontSize: 16.0,
      fontWeight: FontWeight.normal,
    );

  Color get backgroundColor =>
      _backgroundColor ??
      Colors.white;

  TextStyle get tagNormalTextStyle =>
      _tagNormalTextStyle ??
      const TextStyle(
      color: Color(0xFF222222),
      fontSize: 12.0,
    );

  Color get tagNormalBgColor =>
      _tagNormalBgColor ??
      const Color(0xFFF8F8F8);

  TextStyle get tagSelectedTextStyle =>
      _tagSelectedTextStyle ??
      const TextStyle(
      color: Color(0xFF0984F9),
      fontSize: 12.0,
    );

  Color get tagSelectedBgColor =>
      _tagSelectedBgColor ??
      const Color(0xFF0984F9).withAlpha(0x14);

  double get tagRadius =>
      _tagRadius ?? 4.0;

  double get tagSpacing =>
      _tagSpacing ?? 12.0;

  int get preLineTagCount =>
      _preLineTagCount ??
      4;

  double get tagHeight =>
      _tagHeight ?? 32.0;
}
