import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

class TLInputDecoration {
  ///
  // 焦点在输入框内的提示文字
  final String? hintText;
  // 焦点在输入框内的提示文字风格
  final TextStyle? hintStyle;
  // 焦点在输入框内的提示文字最大行数, 默认 1 超过显示..
  final int? hintMaxLines;

  ///
  // 级别最低的border，没有设置其它border时显示的border
  final InputBorder? border;
  final InputBorder? enabledBorder;
  final InputBorder? disabledBorder;
  final InputBorder? focusedBorder;
  final InputBorder? focusedErrorBorder;
  final InputBorder? errorBorder;

  /// 输入框的padding，对内部的文字有效
  final EdgeInsetsGeometry? contentPadding;

  ///
  final bool isCollapsed;

  /// 改变输入框是否为密集型，默认为false，修改为true时，图标及间距会变小，行间距减小。
  final bool? isDense;

  /// 输入框内部是否允许填充 默认 false
  final bool? filled;

  /// 输入框内部填充颜色 filled需要设置为ture才能渲染
  final Color? fillColor;

  /// 显示在输入框内，光标前面的图标，注意icon是在输入框外
  final Widget? prefixIcon;

  ///  PrefixIcon 约束
  final BoxConstraints? prefixIconConstraints;

  /// TLTextField 约束
  final BoxConstraints? constraints;

  InputDecoration get decoration {
    return InputDecoration(
      hintText: hintText ?? "请输入",
      hintStyle: hintStyle ?? TextStyle(color: const Color(0xFF969799), fontSize: 16.sp, height: 1.5),
      hintMaxLines: hintMaxLines,
      border: border,
      enabledBorder: enabledBorder,
      disabledBorder: disabledBorder,
      focusedBorder: focusedBorder,
      focusedErrorBorder: focusedErrorBorder,
      errorBorder: errorBorder,
      contentPadding: contentPadding,
      isCollapsed: isCollapsed,
      isDense: isDense,
      filled: filled,
      fillColor: fillColor,
      prefixIcon: prefixIcon,
      prefixIconConstraints: prefixIconConstraints,
      constraints: constraints,
    );
  }

  ///
  const TLInputDecoration.base({
    this.hintText,
    this.hintStyle,
    this.hintMaxLines,
    this.border,
    this.enabledBorder,
    this.disabledBorder,
    this.focusedBorder,
    this.focusedErrorBorder,
    this.errorBorder,
    this.contentPadding,
    this.isCollapsed = true,
    this.isDense = true,
    this.filled,
    this.fillColor,
    this.prefixIcon,
    this.prefixIconConstraints,
    this.constraints,
  });

  factory TLInputDecoration.spaceInputBorder({
    String? hintText = "请输入",
    TextStyle? hintStyle,
    int? hintMaxLines,
    InputBorder? border = InputBorder.none,
    InputBorder? enabledBorder,
    InputBorder? disabledBorder,
    InputBorder? focusedBorder,
    InputBorder? focusedErrorBorder,
    InputBorder? errorBorder,
    EdgeInsetsGeometry? contentPadding = EdgeInsets.zero,
    bool isCollapsed = true,
    bool isDense = true,
    bool? filled,
    Color? fillColor,
    Widget? prefixIcon,
    BoxConstraints? prefixIconConstraints,
    BoxConstraints? constraints,
  }) {
    return TLInputDecoration.base(
      hintText: hintText,
      hintStyle: hintStyle ?? TextStyle(color: const Color(0xFF969799), fontSize: 16.sp, height: 1.5),
      hintMaxLines: hintMaxLines,
      border: border,
      enabledBorder: enabledBorder,
      disabledBorder: disabledBorder,
      focusedBorder: focusedBorder,
      focusedErrorBorder: focusedErrorBorder,
      errorBorder: errorBorder,
      contentPadding: contentPadding,
      isCollapsed: isCollapsed,
      isDense: isDense,
      filled: filled,
      fillColor: fillColor,
      prefixIcon: prefixIcon,
      prefixIconConstraints: prefixIconConstraints,
      constraints: constraints,
    );
  }

  factory TLInputDecoration.underlineInputBorder({
    String? hintText = "请输入",
    TextStyle? hintStyle,
    int? hintMaxLines,
    InputBorder? border = const UnderlineInputBorder(),
    InputBorder? enabledBorder,
    InputBorder? disabledBorder,
    InputBorder? focusedBorder,
    InputBorder? focusedErrorBorder,
    InputBorder? errorBorder,
    EdgeInsetsGeometry? contentPadding = EdgeInsets.zero,
    bool isCollapsed = true,
    bool isDense = true,
    bool? filled,
    Color? fillColor,
    Widget? prefixIcon,
    BoxConstraints? prefixIconConstraints,
    BoxConstraints? constraints,
  }) {
    return TLInputDecoration.base(
      hintText: hintText,
      hintStyle: hintStyle ?? TextStyle(color: const Color(0xFF969799), fontSize: 16.sp, height: 1.5),
      hintMaxLines: hintMaxLines,
      border: border,
      enabledBorder: enabledBorder,
      disabledBorder: disabledBorder,
      focusedBorder: focusedBorder,
      focusedErrorBorder: focusedErrorBorder,
      errorBorder: errorBorder,
      contentPadding: contentPadding,
      isCollapsed: isCollapsed,
      isDense: isDense,
      filled: filled,
      fillColor: fillColor,
      prefixIcon: prefixIcon,
      prefixIconConstraints: prefixIconConstraints,
      constraints: constraints,
    );
  }

  factory TLInputDecoration.outlineInputBorder({
    String? hintText = "请输入",
    TextStyle? hintStyle,
    int? hintMaxLines,
    InputBorder? border = const OutlineInputBorder(),
    InputBorder? enabledBorder,
    InputBorder? disabledBorder,
    InputBorder? focusedBorder,
    InputBorder? focusedErrorBorder,
    InputBorder? errorBorder,
    EdgeInsetsGeometry? contentPadding,
    bool isCollapsed = true,
    bool isDense = true,
    bool? filled,
    Color? fillColor,
    Widget? prefixIcon,
    BoxConstraints? prefixIconConstraints,
    BoxConstraints? constraints,
  }) {
    return TLInputDecoration.base(
      hintText: hintText,
      hintStyle: hintStyle ?? TextStyle(color: const Color(0xFF969799), fontSize: 16.sp, height: 1.5),
      hintMaxLines: hintMaxLines,
      border: border,
      enabledBorder: enabledBorder,
      disabledBorder: disabledBorder,
      focusedBorder: focusedBorder,
      focusedErrorBorder: focusedErrorBorder,
      errorBorder: errorBorder,
      contentPadding: contentPadding ?? EdgeInsets.symmetric(vertical: 10.w, horizontal: 8.h),
      isCollapsed: isCollapsed,
      isDense: isDense,
      filled: filled,
      fillColor: fillColor,
      prefixIcon: prefixIcon,
      prefixIconConstraints: prefixIconConstraints,
      constraints: constraints,
    );
  }

  factory TLInputDecoration.filledInputBorder({
    String? hintText = "请输入",
    TextStyle? hintStyle,
    int? hintMaxLines,
    InputBorder? border = InputBorder.none,
    InputBorder? enabledBorder,
    InputBorder? disabledBorder,
    InputBorder? focusedBorder,
    InputBorder? focusedErrorBorder,
    InputBorder? errorBorder,
    EdgeInsetsGeometry? contentPadding,
    bool isCollapsed = true,
    bool isDense = true,
    bool? filled = true,
    Color? fillColor = const Color(0xFFF7F8FA),
    Widget? prefixIcon,
    BoxConstraints? prefixIconConstraints,
    BoxConstraints? constraints,
  }) {
    return TLInputDecoration.base(
      hintText: hintText,
      hintStyle: hintStyle ?? TextStyle(color: const Color(0xFF969799), fontSize: 16.sp, height: 1.5),
      hintMaxLines: hintMaxLines,
      border: border,
      enabledBorder: enabledBorder,
      disabledBorder: disabledBorder,
      focusedBorder: focusedBorder,
      focusedErrorBorder: focusedErrorBorder,
      errorBorder: errorBorder,
      contentPadding: contentPadding ?? EdgeInsets.symmetric(vertical: 10.w, horizontal: 8.h),
      isCollapsed: isCollapsed,
      isDense: isDense,
      filled: filled,
      fillColor: fillColor,
      prefixIcon: prefixIcon,
      prefixIconConstraints: prefixIconConstraints,
      constraints: constraints,
    );
  }
}
