import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class TLTextFieldStyle {
  const TLTextFieldStyle({
    this.style,
    this.decoration,
    this.inputFormatters,
    this.enabled,
    this.readOnly = false,
    this.keyboardType,
    this.textInputAction,
    this.maxLines,
    this.minLines,
    this.maxLength,
    this.maxLengthEnforcement,
    this.showCursor,
    this.cursorWidth = 2,
    this.cursorHeight,
    this.cursorRadius,
    this.cursorColor,
  });

  /// 文本样式
  final TextStyle? style;

  /// 用于控制TextField的外观显示，如提示文本、背景颜色、边框等
  final InputDecoration? decoration;

  /// 用于指定输入格式；当用户输入内容改变时，会根据指定的格式来校验。
  final List<TextInputFormatter>? inputFormatters;

  /// 输入框是否禁用；如果为false，则输入框会被禁用，禁用状态不接收输入和手势事件，同时显示禁用态样式（在其decoration中定义）。
  final bool? enabled;

  //是否自读,true后不可编辑；接受onTap等手势事件
  final bool readOnly;

  /// 键盘类型
  //  TextInputType.text              文本输入键盘
  //  TextInputType.multiline         多行文本，需和maxLines配合使用(设为null或大于1)
  //  TextInputType.number            数字键盘
  //  TextInputType.numberWithOptions(decimal: true) 数字键盘；会显示 .
  //  TextInputType.phone             优化后的电话号码输入键盘；会弹出数字键盘并显示“* #”
  //  TextInputType.datetime          优化后的日期输入键盘；会显示“: -”
  //  TextInputType.emailAddress      优化后的电子邮件地址；会显示“@ .”
  //  TextInputType.url               优化后的url输入键盘； 会显示“/ .”
  final TextInputType? keyboardType;

  /// 设置键盘上enter键的显示内容
  //  TextInputAction.none             默认回车符号
  final TextInputAction? textInputAction;

  /// 不是允许输入的最大行数,指的是输入框内可显示的高度是几行，超过设定行数后，scroll滚动显示。系统设置默认1行，这里null则表示无限制(多行)
  final int? maxLines;

  /// 最小行数且设置时最小值为1
  final int? minLines;

  /// 输入框中允许的最大字符数.设置此项会让TextField右下角有一个输入数量的统计字符串
  final int? maxLength;

  /// maxLengthEnforcement
  final MaxLengthEnforcement? maxLengthEnforcement;

  /// 是否显示光标
  final bool? showCursor;

  /// 光标宽度
  final double cursorWidth;

  /// 光标高度
  final double? cursorHeight;

  /// 光标弧度
  final Radius? cursorRadius;

  /// 光标颜色
  final Color? cursorColor;
}
