import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:tengits_widgets/tengits_widgets.dart';

class TLCardTextField extends StatefulWidget {
  const TLCardTextField({
    Key? key,
    this.defaultValue,
    this.cardTextFieldStyle = const TLCardTextFieldStyle(),
    this.cardTextFieldEvent,
  }) : super(key: key);

  /// 文本默认值
  final String? defaultValue;

  /// 风格配置
  final TLCardTextFieldStyle cardTextFieldStyle;

  /// Builders集 手势事件
  final TLCardTextFieldEvent? cardTextFieldEvent;

  @override
  State<TLCardTextField> createState() => _TLCardTextFieldState();
}

class _TLCardTextFieldState extends State<TLCardTextField> {
  ///
  late String? _defaultValue;
  late TLCardTextFieldStyle _cardTextFieldStyle;
  late TLCardTextFieldEvent? _cardTextFieldEvent;

  late int? _length;

  @override
  void initState() {
    ///
    _defaultValue = widget.defaultValue;
    _length = _defaultValue?.length;

    ///
    _cardTextFieldStyle = widget.cardTextFieldStyle;

    ///
    _cardTextFieldEvent = widget.cardTextFieldEvent;

    super.initState();
  }

  @override
  void didUpdateWidget(TLCardTextField oldWidget) {
    ///
    if (oldWidget.defaultValue != widget.defaultValue) {
      ///
      _defaultValue = widget.defaultValue;
      _length = _defaultValue?.length;
    }

    ///
    if (oldWidget.cardTextFieldStyle != widget.cardTextFieldStyle) {
      ///
      _cardTextFieldStyle = widget.cardTextFieldStyle;
    }

    ///
    if (oldWidget.cardTextFieldEvent != widget.cardTextFieldEvent) {
      ///
      _cardTextFieldEvent = widget.cardTextFieldEvent;
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: _cardTextFieldStyle.margin, // 外边距
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          GestureDetector(
            onTap: _cardTextFieldEvent?.onTap,
            child: Card(
              color: _cardTextFieldStyle.color, // 背景色
              shadowColor: _cardTextFieldStyle.shadowColor, // 阴影颜色
              elevation: _cardTextFieldStyle.elevation, // 阴影高度
              shape: _cardTextFieldStyle.shape ?? RoundedRectangleBorder(borderRadius: BorderRadius.circular(_cardTextFieldStyle.radius ?? 4.r)), // 边框
              borderOnForeground: false, // 是否在 child 前绘制 border，默认为 true
              margin: EdgeInsets.zero,
              clipBehavior: _cardTextFieldStyle.clipBehavior, // 剪切模式
              child: TLTextField(
                defaultValue: _defaultValue,
                textFieldStyle: TLTextFieldStyle(
                  style: _cardTextFieldStyle.textFieldStyle?.style ?? TextStyle(color: const Color(0xFF323233), fontSize: 15.sp, height: 1.5),
                  decoration: TLInputDecoration.spaceInputBorder(
                    hintText: _cardTextFieldStyle.textFieldStyle?.decoration?.hintText ?? (_cardTextFieldStyle.textFieldStyle?.enabled == false ? _cardTextFieldStyle.hintDisabledText ?? "" : "请输入"),
                    hintStyle: _cardTextFieldStyle.textFieldStyle?.decoration?.hintStyle ?? TextStyle(color: const Color(0xFF969799), fontSize: 16.sp, height: 1.5),
                    hintMaxLines: _cardTextFieldStyle.textFieldStyle?.decoration?.hintMaxLines,
                    contentPadding: _cardTextFieldStyle.textFieldStyle?.decoration?.contentPadding ?? EdgeInsets.symmetric(vertical: 10.w, horizontal: 8.h), // 内边距
                    constraints: _cardTextFieldStyle.textFieldStyle?.decoration?.constraints ??
                        BoxConstraints(
                          minHeight: _cardTextFieldStyle.minHeight ?? 48.h,
                          maxHeight: _cardTextFieldStyle.maxHeight ?? double.infinity,
                        ),
                  ).decoration,
                  inputFormatters: _cardTextFieldStyle.textFieldStyle?.inputFormatters ??
                      TLTextInputFormatter.maxLengthInputFormatters(maxLength: _cardTextFieldStyle.textFieldStyle?.maxLength ?? _cardTextFieldStyle.inputMaxLength),
                  enabled: _cardTextFieldStyle.textFieldStyle?.enabled,
                  readOnly: _cardTextFieldStyle.textFieldStyle?.readOnly ?? false,
                  keyboardType: _cardTextFieldStyle.textFieldStyle?.keyboardType ?? TextInputType.text,
                  textInputAction: _cardTextFieldStyle.textFieldStyle?.textInputAction,
                  maxLines: _cardTextFieldStyle.textFieldStyle?.maxLines,
                  minLines: _cardTextFieldStyle.textFieldStyle?.minLines,
                  // maxLength: _cardTextFieldStyle.textFieldStyle?.maxLength, 注销后采用下面代码的 最大长度自定义标签计数
                  maxLengthEnforcement: _cardTextFieldStyle.textFieldStyle?.maxLengthEnforcement,
                  showCursor: _cardTextFieldStyle.textFieldStyle?.showCursor,
                  cursorWidth: _cardTextFieldStyle.textFieldStyle?.cursorWidth ?? 2,
                  cursorHeight: _cardTextFieldStyle.textFieldStyle?.cursorHeight,
                  cursorRadius: _cardTextFieldStyle.textFieldStyle?.cursorRadius,
                  cursorColor: _cardTextFieldStyle.textFieldStyle?.cursorColor,
                ),
                textFieldEvent: _cardTextFieldEvent?.textFieldEvent,
                textEditingController: (TextEditingController textEditingController) {
                  ///
                  textEditingController.addListener(() {
                    /// 解决在build的时候不允许调用setState() 或 markNeedsBuild()
                    WidgetsBinding.instance?.addPostFrameCallback((timeStamp) {
                      if (mounted) {
                        setState(() {
                          _length = textEditingController.text.length;
                        });
                      }
                    });
                  });
                },
              ),
            ),
          ),
          Visibility(
            visible: _cardTextFieldStyle.textFieldStyle?.maxLength != null,
            child: Container(
              alignment: Alignment.centerRight,
              padding: EdgeInsets.symmetric(vertical: 5.h),
              child: Text(
                "${_length ?? 0}/${_cardTextFieldStyle.textFieldStyle?.maxLength ?? 0}",
                style: const TextStyle(color: Color(0xFFC8C9CC)),
              ),
            ),
          ),
        ],
      ),
    );
  }
}
