import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:tengits_widgets/text_input/src/event/text_field_event.dart';
import 'package:tengits_widgets/text_input/src/style/text_field_input_decoration.dart';
import 'package:tengits_widgets/text_input/src/style/text_field_style.dart';

class TLTextField extends StatefulWidget {
  const TLTextField({
    Key? key,
    this.defaultValue,
    this.textFieldStyle = const TLTextFieldStyle(),
    this.textFieldEvent,
    this.textEditingController,
  }) : super(key: key);

  /// 文本默认值
  final String? defaultValue;

  /// 风格配置
  final TLTextFieldStyle textFieldStyle;

  /// Builders集 手势事件
  final TLTextFieldEvent? textFieldEvent;

  ///
  final void Function(TextEditingController textEditingController)? textEditingController;

  @override
  State<TLTextField> createState() => _TLTextFieldState();
}

class _TLTextFieldState extends State<TLTextField> {
  ///
  late final TextEditingController _textEditingController;
  late TLTextFieldStyle _textFieldStyle;
  late TLTextFieldEvent? _textFieldEvent;

  _handleControllerChanged() {
    _textFieldEvent?.onChanged?.call(_textEditingController.text);
  }

  @override
  void initState() {
    ///
    _textEditingController = TextEditingController(text: widget.defaultValue);

    ///
    _textFieldStyle = widget.textFieldStyle;

    ///
    _textFieldEvent = widget.textFieldEvent;

    ///
    _textEditingController.addListener(_handleControllerChanged);

    ///
    TextSelection position = _textEditingController.selection.copyWith();
    if (_textEditingController.isSelectionWithinTextBounds(position)) {
      _textEditingController.selection = position;
    }

    ///
    widget.textEditingController?.call(_textEditingController);

    super.initState();
  }

  @override
  void didUpdateWidget(TLTextField oldWidget) {
    ///
    if (oldWidget.defaultValue != widget.defaultValue) {
      ///
      _textEditingController.text = widget.defaultValue ?? "";
    }

    ///
    if (oldWidget.textFieldStyle != widget.textFieldStyle) {
      ///
      _textFieldStyle = widget.textFieldStyle;
    }

    ///
    if (oldWidget.textFieldEvent != widget.textFieldEvent) {
      ///
      _textFieldEvent = widget.textFieldEvent;
    }

    ///
    TextSelection position = _textEditingController.selection.copyWith();
    if (_textEditingController.isSelectionWithinTextBounds(position)) {
      _textEditingController.selection = position;
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    ///
    _textEditingController.removeListener(_handleControllerChanged);

    ///
    _textEditingController.dispose();

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return TextField(
      key: widget.key,
      controller: _textEditingController, // TextEditingController 是 TextField 的控制类，可以控制 TextField 的编辑。
      focusNode: null, // 用于控制TextField是否占有当前键盘的输入焦点。
      style: _textFieldStyle.style ?? TextStyle(color: const Color(0xFF323233), fontSize: 15.sp, height: 1.5),
      decoration: _textFieldStyle.decoration ?? TLInputDecoration.spaceInputBorder().decoration,
      inputFormatters: _textFieldStyle.inputFormatters,
      enabled: _textFieldStyle.enabled,
      readOnly: _textFieldStyle.readOnly,
      keyboardType: _textFieldStyle.keyboardType,
      textInputAction: _textFieldStyle.textInputAction,
      maxLines: _textFieldStyle.maxLines,
      minLines: _textFieldStyle.minLines,
      maxLength: _textFieldStyle.maxLength,
      maxLengthEnforcement: _textFieldStyle.maxLengthEnforcement,
      showCursor: _textFieldStyle.showCursor,
      cursorWidth: _textFieldStyle.cursorWidth,
      cursorHeight: _textFieldStyle.cursorHeight,
      cursorRadius: _textFieldStyle.cursorRadius,
      cursorColor: _textFieldStyle.cursorColor,
      // onChanged: _textFieldEvent?.onChanged,  注销后采用 _textEditingController.addListener 监听回调触发
      onSubmitted: _textFieldEvent?.onSubmitted,
      onTap: _textFieldEvent?.onTap,
    );
  }
}
